"""
todo.py

Usage:
    todo.py <project-name> add <task>
    todo.py add <task>
    todo.py <project-name> del <id>
    todo.py del <id>
    todo.py <project-name> list
    todo.py list
    todo.py (-o PATH | --change-path=PATH) [--move]
    todo.py (-s | --show-path)

Options:
    -h --help                   Shows the default help screen (this screen!)
    -o PATH --change-path=PATH  Changes the path of the output file
    -s --show-path              Shows the path of the output file
"""

import os
import csv
import json
import shutil

from docopt import docopt
from tabulate import tabulate
import arrow

import _version

filename = "tasks.csv"


class Tasks(object):
    def __init__(self):
        self.path = read_config()
        self.file_path = os.path.join(self.path, filename)

    def add(self, project, task):
        if not os.path.isfile(self.file_path):
            self.make_file()

        _id = self.final_id(project) + 1
        date = arrow.now().format('YYYY-MM-DD HH:mm:ss ZZ')

        with open(self.file_path, "ab") as tasks_file:
            csv_writer = csv.writer(tasks_file)
            data = [project, _id, date, task]
            csv_writer.writerow(data)

    def delete(self, project, id_list):
        ids = []
        deleted = False
        for i in id_list.split(','):
            try:
                int(i)
                ids.append(i)
            except ValueError:
                if i:
                    print "ID {} is not valid".format(i)
        if not ids:
            raise Exception("Enter a valid id(s.)")
        try:
            with open(self.file_path, "rb") as tasks_file:
                csv_reader = csv.reader(tasks_file)
                new_data = []
                for row in csv_reader:
                    if row[1] not in ids:
                        new_data.append(row)
                    else:
                        if row[0] not in [project, "Project"]:
                            new_data.append(row)
                        else:
                            deleted = True
        except IOError:
            raise Exception("{} was not found".format(filename))
        if not deleted:
            raise Exception("ID entered was not found or project name is invalid.")
        if new_data:  # To avoid the risk of mistakenly overwriting the tasks file
            with open(self.file_path, "wb") as tasks_file:
                csv_writer = csv.writer(tasks_file)
                csv_writer.writerows(new_data)
            print "Selected ID's successfully deleted."

    def list(self, project):
        listed = False
        try:
            with open(self.file_path, "rb") as tasks_file:
                csv_reader = csv.reader(tasks_file)
                data = []
                for index, row in enumerate(csv_reader):
                    if index > 0 and row[0] == project:
                        humanized_date = arrow.get(row[2], 'YYYY-MM-DD HH:mm:ss ZZ').humanize()
                        new_row = [row[0].capitalize(), row[1], humanized_date, row[3]]
                        data.append(new_row)
                        listed = True
                if data:
                    print tabulate(data, headers=["Project", "ID", "Created", "Task"], tablefmt="psql",
                                   stralign="center", numalign="center")
        except IOError:
            raise Exception("{} was not found".format(filename))
        if not listed:
            raise Exception("Project name invalid")

    def make_file(self):
        headers = ["Project", "id", "Created", "Task"]
        with open(self.file_path, "ab") as tasks_file:
            csv_writer = csv.writer(tasks_file)
            csv_writer.writerow(headers)

    def final_id(self, project):
        rows = []
        with open(self.file_path, "rb") as tasks_file:
            csv_reader = csv.reader(tasks_file)
            for index, row in enumerate(csv_reader):
                if index > 0:
                    if row[0] == project:
                        rows.append(row)
            if not rows:
                return 0  # First run
            try:
                _id = int(rows[-1][1])  # last row, first element represents the 'id'
                return _id
            except ValueError:
                return 0  # id doesn't exist

    @staticmethod
    def move(path):
        old_path = os.path.join(read_config(), filename)
        new_path = os.path.join(path, filename)
        shutil.move(old_path, new_path)


def arguments_processor(args):
    tasks = Tasks()
    project = "general"

    if args['<project-name>']:
        project = args['<project-name>'].lower()

    if args['add']:
        task = args['<task>']
        tasks.add(project, task)

    if args['del']:
        _id = args['<id>']
        tasks.delete(project, _id)

    if args['list']:
        tasks.list(project)

    if args['--show-path']:
        print "Current path is : {}".format(read_config())


def read_config():
    with open('config.txt', 'r') as config_file:
        path = json.loads(config_file.read())['path']  # only the variable "path" exists in the config file
        return path


def write_config(path=None):
    if path is None:
        path = os.getcwd()
    with open('config.txt', 'w') as config_file:
        data = {'path': path}
        json.dump(data, config_file)


def main():
    arguments = docopt(__doc__, version=_version.__version_info__)
    if not os.path.isfile("config.txt"):
        write_config(arguments['--change-path'])  # By default --change-path=None
        print "Config file successfully created"
    elif arguments['--change-path']:
        new_path = arguments['--change-path']
        Tasks.move(new_path)
        write_config(new_path)
        print "Config file successfully modified, updated path " + read_config()
    arguments_processor(arguments)

if __name__ == '__main__':
    main()
